//
//  ButtonSettings.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 04.09.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import UIKit

class ButtonSettings: NSObject{
    private var btn: UIButton!
    private var btnManually: UIButton!
    private var btnAutomatic: UIButton!
    private var state: String!
    private var pin: Int!
    private var psService: PowerSocketService!
    private var isManually: Bool!
    private var btnName: String!
    
    init(button: UIButton, manually: UIButton, auto: UIButton, state: String, pin: Int, isManually: Bool, name: String) {
        self.btn = button
        self.state = state
        self.pin = pin
        self.btnManually = manually
        self.btnAutomatic = auto
        self.isManually = isManually
        self.btnName = name
        //btn.addTarget(btn, action: #selector(buttonAction), for: .touchUpInside)
        
    }
    
    func getState() -> String {
        return self.state
    }
    
    func getBtn() -> UIButton {
        return self.btn
    }
    
    func getManuallyButton() -> UIButton {
        return self.btnManually
    }
    
    func getAutomaticButton() -> UIButton {
        return self.btnAutomatic
    }
    
    func getIsManually() -> Bool {
        return self.isManually
    }
    
    func getPin() -> Int {
        return self.pin
    }
    
    func setState(newState: String) {
        self.state = newState
    }
    
    func setPowerSocketService(service: PowerSocketService) {
        self.psService = service
    }
    
    func setButtonState() {
        if self.state == "ON" {
            btn.setTitle(String(self.pin) + ". " + self.btnName + " is ON", for: .normal)
            btn.backgroundColor = UIColor.green
        } else if self.state == "OFF" {
            btn.setTitle(String(self.pin) + ". " + self.btnName + " is OFF", for: .normal)
            btn.backgroundColor = UIColor.red
        }
    }
    
    func switchState() {
        if self.state == "ON" {
            self.state = "OFF"
            btn.setTitle(btn.titleLabel?.text?.replacingOccurrences(of: "ON", with: "OFF"), for: .normal)
            btn.backgroundColor = UIColor.red
            // set PowerSocketPin in Service
            psService.setPowerSocketPin(pin: self.pin, state: self.state)
        } else if self.state == "OFF" {
            self.state = "ON"
            btn.setTitle(btn.titleLabel?.text?.replacingOccurrences(of: "OFF", with: "ON"), for: .normal)
            btn.backgroundColor = UIColor.green
            psService.setPowerSocketPin(pin: self.pin, state: self.state)
        }
    }
    
    func switchToOn() {
        if self.state == "OFF" {
            self.state = "ON"
            btn.setTitle(btn.titleLabel?.text?.replacingOccurrences(of: "OFF", with: "ON"), for: .normal)
            btn.backgroundColor = UIColor.green
        }
    }
    
    func switchToOff() {
        if self.state == "ON" {
            self.state = "OFF"
            btn.setTitle(btn.titleLabel?.text?.replacingOccurrences(of: "ON", with: "OFF"), for: .normal)
            btn.backgroundColor = UIColor.red
        }
    }
    
    func switchRadioButtons(isMan: Bool!) {
        if isMan {
            if !isManually {
                isManually = true
                btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .normal)
                btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .highlighted)
                btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .disabled)
                btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .normal)
                btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .highlighted)
                btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .disabled)
                btn.isEnabled = true
            }
        } else if !isMan {
            if isManually {
                isManually = false
                btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .normal)
                btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .highlighted)
                btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .disabled)
                btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .normal)
                btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .highlighted)
                btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .disabled)
                btn.isEnabled = false
            }
        }
    }
    
    func setButtonName(newName: String!) {
        btn.setTitle(btn.titleLabel?.text?.replacingOccurrences(of: self.btnName, with: newName), for: .normal)
        self.btnName = newName
    }
    
    func buttonAction(_ sender: UIButton!) {
        //print("Button " + String(self.pin) + " was tapped. State: " + self.state + ". Switch state!")
        switchState()
    }
    
    func radioButtonAction(_ sender: UIButton!, isManually: Bool!) {
        switchRadioButtons(isMan: isManually)
    }
    
    func changeToSmallButtonImages() {
        if isManually {
            btnManually.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .normal)
            btnManually.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .highlighted)
            btnManually.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .disabled)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected30x30"), for: .normal)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected30x30"), for: .highlighted)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected30x30"), for: .disabled)
        } else if !isManually {
            btnManually.setImage(UIImage(named: "RadioButtonUnselected30x30"), for: .normal)
            btnManually.setImage(UIImage(named: "RadioButtonUnselected30x30"), for: .highlighted)
            btnManually.setImage(UIImage(named: "RadioButtonUnselected430x30"), for: .disabled)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .normal)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .highlighted)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected30x30"), for: .disabled)
        }
    }
    
    func changeToBigButtonImages() {
        if isManually {
            btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .normal)
            btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .highlighted)
            btnManually.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .disabled)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .normal)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .highlighted)
            btnAutomatic.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .disabled)
        } else if !isManually {
            btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .normal)
            btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .highlighted)
            btnManually.setImage(UIImage(named: "RadioButtonUnselected40x40"), for: .disabled)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .normal)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .highlighted)
            btnAutomatic.setImage(UIImage(named: "RadioButtonSelected40x40"), for: .disabled)
        }
    }
    
}
