//
//  MainController.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 03.09.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import UIKit

extension StringProtocol {
    subscript(offset: Int) -> Element {
        return self[index(startIndex, offsetBy: offset)]
    }
}

class MainController: UIViewController {

    private var deviceDescription: TableViewDataModelItem!
    private var bluetoothManager: BluetoothManager!
    private var psService: PowerSocketService!
    private var ctService: CurrentTimeService!
    private var tsService: TimeSettingsService!
    private let fileRadioStates = "SavedRadioStates.txt"
    private var fileRadioStatesURL: URL? = nil
    private let fileButtonNames = "SavedButtonNames.txt"
    private var fileButtonNamesURL : URL? = nil
    
    private var allButtonSettings = [ButtonSettings?](repeating: nil, count:7)
    @IBOutlet weak var btnAll_ON: UIButton!
    @IBOutlet weak var btnAll_OFF: UIButton!
    @IBOutlet weak var btnDisconnect: UIButton!
    @IBOutlet weak var btnSettings: UIButton!
    @IBOutlet weak var btnPS_1: UIButton!
    @IBOutlet weak var btnPS1_Man: UIButton!
    @IBOutlet weak var btnPS1_Auto: UIButton!
    @IBOutlet weak var btnPS_2: UIButton!
    @IBOutlet weak var btnPS2_Man: UIButton!
    @IBOutlet weak var btnPS2_Auto: UIButton!
    @IBOutlet weak var btnPS_3: UIButton!
    @IBOutlet weak var btnPS3_Man: UIButton!
    @IBOutlet weak var btnPS3_Auto: UIButton!
    @IBOutlet weak var btnPS_4: UIButton!
    @IBOutlet weak var btnPS4_Man: UIButton!
    @IBOutlet weak var btnPS4_Auto: UIButton!
    @IBOutlet weak var btnPS_5: UIButton!
    @IBOutlet weak var btnPS5_Man: UIButton!
    @IBOutlet weak var btnPS5_Auto: UIButton!
    @IBOutlet weak var btnPS_6: UIButton!
    @IBOutlet weak var btnPS6_Man: UIButton!
    @IBOutlet weak var btnPS6_Auto: UIButton!
    @IBOutlet weak var btnPS_7: UIButton!
    @IBOutlet weak var btnPS7_Man: UIButton!
    @IBOutlet weak var btnPS7_Auto: UIButton!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        print("MainController did load")
        
        if let dir = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first {
            fileRadioStatesURL = dir.appendingPathComponent(fileRadioStates)
            fileButtonNamesURL = dir.appendingPathComponent(fileButtonNames)
        }
        
        instanciateButtons()
        
        NotificationCenter.default.addObserver(self, selector: #selector(connectionCompleteNotification), name: RCNotification.ConnectionComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(PowerSocketServiceScanCompleteNotification), name: RCNotification.PowerSocketServiceScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(CurrentTimeServiceScanCompleteNotification), name: RCNotification.CurrentTimeServiceScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(TimeSettingsServiceScanCompleteNotification), name: RCNotification.TimeSettingsServiceScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(PowerSocketCharacteristicScanCompleteNotification), name: RCNotification.PowerSocketCharacteristicsScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(CurrentTimeCharacteristicScanCompleteNotification), name: RCNotification.CurrentTimeCharacteristicsScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(TimeSettingsCharacteristicScanCompleteNotification), name: RCNotification.TimeSettingsCharacteristicsScanComplete, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(readCharacteristicsNotification(notification:)), name: RCNotification.PowerSocketReadCharacteristicValue, object: nil)
        NotificationCenter.default.addObserver(self, selector: #selector(buttonNamesChangedNotification), name: RCNotification.ButtonNamesChanged, object: nil)
        
        btnAll_ON.isEnabled = false
        btnAll_OFF.isEnabled = false
        btnDisconnect.isEnabled = false
        btnSettings.isEnabled = false
        for i in 0..<7 {
            allButtonSettings[i]?.getManuallyButton().isEnabled = false
            allButtonSettings[i]?.getAutomaticButton().isEnabled = false
            allButtonSettings[i]?.getBtn().isEnabled = false
        }
        // Do any additional setup after loading the view.
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        print("MainController did appear")
        
    }
    
    override func viewDidDisappear(_ animated: Bool) {
        super.viewDidDisappear(animated)
        saveRadioStates()
        print("MainController view did Disappear")
    }

    func backToScanController() {
        deviceDescription = nil
        psService = nil
        ctService = nil
        tsService = nil
    }
    
    func saveRadioStates() {
        var text = ""
        for i in allButtonSettings {
            if (i?.getIsManually())! {
                text += "0"
            } else {
                text += "1"
            }
        }
        do {
            try text.write(to: fileRadioStatesURL!, atomically: false, encoding: .utf8)
        } catch {
            print("Error with writing radio States")
        }
    }
    
    func readRadioState() -> String! {
        var text = ""
        do {
            text = try String(contentsOf: fileRadioStatesURL!, encoding: .utf8)
        } catch {
            print("Error with reading Radio States")
        }
        return text
    }
    
    func readButtonNames() -> (n1: String?, n2: String?, n3: String?, n4: String?, n5: String?, n6: String?, n7: String?) {
        var text = ""
        do {
            text = try String(contentsOf: fileButtonNamesURL!, encoding: .utf8)
        } catch {
            print("Error with reading Radio States")
        }
        if text == "" {
            return ("Power Socket", "Power Socket", "Power Socket", "Power Socket", "Power Socket", "Power Socket", "Power Socket")
        } else {
            let allNames = text.components(separatedBy: ";")
            return (allNames[0], allNames[1], allNames[2], allNames[3], allNames[4], allNames[5], allNames[6])
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    public func setDeviceDescription(item: TableViewDataModelItem) {
        self.deviceDescription = item
    }

    func instanciateButtons() {
        print(readButtonNames())
        let btn_PS1 = ButtonSettings(button: btnPS_1, manually: btnPS1_Man, auto: btnPS1_Auto, state: "OFF", pin: 1, isManually: false, name: readButtonNames().n1!)
        let btn_PS2 = ButtonSettings(button: btnPS_2, manually: btnPS2_Man, auto: btnPS2_Auto, state: "OFF", pin: 2, isManually: false, name: readButtonNames().n2!)
        let btn_PS3 = ButtonSettings(button: btnPS_3, manually: btnPS3_Man, auto: btnPS3_Auto, state: "OFF", pin: 3, isManually: false, name: readButtonNames().n3!)
        let btn_PS4 = ButtonSettings(button: btnPS_4, manually: btnPS4_Man, auto: btnPS4_Auto, state: "OFF", pin: 4, isManually: false, name: readButtonNames().n4!)
        let btn_PS5 = ButtonSettings(button: btnPS_5, manually: btnPS5_Man, auto: btnPS5_Auto, state: "OFF", pin: 5, isManually: false, name: readButtonNames().n5!)
        let btn_PS6 = ButtonSettings(button: btnPS_6, manually: btnPS6_Man, auto: btnPS6_Auto, state: "OFF", pin: 6, isManually: false, name: readButtonNames().n6!)
        let btn_PS7 = ButtonSettings(button: btnPS_7, manually: btnPS7_Man, auto: btnPS7_Auto, state: "OFF", pin: 7, isManually: false, name: readButtonNames().n7!)
        
        allButtonSettings[0] = btn_PS1
        allButtonSettings[1] = btn_PS2
        allButtonSettings[2] = btn_PS3
        allButtonSettings[3] = btn_PS4
        allButtonSettings[4] = btn_PS5
        allButtonSettings[5] = btn_PS6
        allButtonSettings[6] = btn_PS7
        
    }
    
    public func setBluetoothManager(manager: BluetoothManager) {
        self.bluetoothManager = manager
    }
    
    @objc func connectionCompleteNotification() {
        print("Connection complete Notification was sent")
        bluetoothManager.discoverServices()
    }
    
    @objc func PowerSocketServiceScanCompleteNotification() {
        print("Discovered Power Socket Service Complete Notification sent")
        bluetoothManager.discoverPowerSocketCharacteristics()
    }
    
    @objc func CurrentTimeServiceScanCompleteNotification() {
        print("Discovered Current Time Service Complete Notification sent")
        bluetoothManager.discoverCurrentTimeCharacteristics()
    }
    
    @objc func TimeSettingsServiceScanCompleteNotification() {
        print("Discovered Time Settings Service Complete Notiication sent")
        bluetoothManager.discoverTimeSettingsCharacteristics()
    }
    
    @objc func PowerSocketCharacteristicScanCompleteNotification() {
        print("Discovered Characteristics for PowerSocketService Complete Notification sent")
        psService = bluetoothManager.getPsService()
        for btn in allButtonSettings {
            btn?.setPowerSocketService(service: psService)
        }
        bluetoothManager.readPowerSocketCharacteristic()
    }
    
    @objc func CurrentTimeCharacteristicScanCompleteNotification() {
        ctService = bluetoothManager.getCTService()
        print("Discovered Characteristics for CurrentTimeService Complete Notification sent")
        ctService.setCurrentTime()
    }
    
    @objc func TimeSettingsCharacteristicScanCompleteNotification() {
        tsService = bluetoothManager.getTSService()
        print("Discovered Characteristics for TimeSettingsService Complete Notification sent")
        setRadioStates()
    }
    
    @objc func readCharacteristicsNotification(notification: NSNotification) {
        print("read Characteristics Notification sent")
        let notification : [AnyHashable: Any] = notification.userInfo!
        let value = notification[AnyHashable("readValue")] as! Data
        setButtonStates(values: value)
        
        btnDisconnect.isEnabled = true
        btnSettings.isEnabled = true
        for i in 0..<7 {
            allButtonSettings[i]?.getManuallyButton().isEnabled = true
            allButtonSettings[i]?.getAutomaticButton().isEnabled = true
        }
    }
    
    @objc func buttonNamesChangedNotification() {
        print("button Names changed")
        allButtonSettings[0]?.setButtonName(newName: readButtonNames().n1)
        allButtonSettings[1]?.setButtonName(newName: readButtonNames().n2)
        allButtonSettings[2]?.setButtonName(newName: readButtonNames().n3)
        allButtonSettings[3]?.setButtonName(newName: readButtonNames().n4)
        allButtonSettings[4]?.setButtonName(newName: readButtonNames().n5)
        allButtonSettings[5]?.setButtonName(newName: readButtonNames().n6)
        allButtonSettings[6]?.setButtonName(newName: readButtonNames().n7)
    }
    
    func setButtonStates(values: Data) {
        var bStates = [Int?](repeating: nil, count: 7)
        
        for i:Int in 0..<14 {
            if i % 2 == 1 {
                bStates[i / 2] = Int(values[i])
            }
        }
        
        for i: Int in 0..<7 {
            let val: Int = bStates[i]!
            print(val)
            var state: String? = nil
            if val == 0 {
                state = "OFF"
            } else {
                state = "ON"
            }
            allButtonSettings[i]?.setState(newState: state!)
            allButtonSettings[i]?.setButtonState()
        }
        checkButtonStates()
    }
    
    func setRadioStates() {
        let radioStates = readRadioState()
        if radioStates != "" {
            for i in 0..<allButtonSettings.count {
                print(radioStates![i])
                if radioStates![i] == "0" {
                    allButtonSettings[i]?.switchRadioButtons(isMan: true)
                } else {
                    allButtonSettings[i]?.switchRadioButtons(isMan: false)
                }
            }
        }
        checkForRadioSettings()
    }
    
    /***
     Checks if the 7 Buttons have the same state. If true, color the right Button above (All_ON or All_OFF)
     If one of them have another state, clear the Buttons above
    */
    func checkButtonStates() {
        let prevState : String? = allButtonSettings[0]?.getState()
        var hasChanged : Bool = false
        for i in allButtonSettings {
            if i?.getState() == prevState {
                hasChanged = false
            } else {
                hasChanged = true
                break
            }
        }
        
        if hasChanged == false {
            if prevState == "ON" {
                btnAll_ON.backgroundColor = UIColor.green
                btnAll_OFF.backgroundColor = .clear
            } else if prevState == "OFF" {
                btnAll_OFF.backgroundColor = UIColor.red
                btnAll_ON.backgroundColor = .clear
            }
        } else {
            btnAll_ON.backgroundColor = .clear
            btnAll_OFF.backgroundColor = .clear
        }
    }
    
    /***
     Checks if, on one of the Radio Buttons, the Automatic is selected
     If true, disable ALL_ON and ALL_OFF Button
    */
    func checkForRadioSettings() {
        var isOneAuto : Bool = false
        var timePins = [Int?](repeating: nil, count:7)
        
        for i in allButtonSettings {
            if !(i?.getIsManually())! {
                i?.getBtn().isEnabled = false
                timePins[(i?.getPin())! - 1] = (i?.getPin())! - 1
            } else {
                i?.getBtn().isEnabled = true
                timePins[(i?.getPin())! - 1] = 8
            }
        }
        
        for i in allButtonSettings {
            if !(i?.getIsManually())! {
                btnAll_ON.isEnabled = false
                btnAll_OFF.isEnabled = false
                isOneAuto = true
                break
            }
        }
        
        if !isOneAuto {
            btnAll_ON.isEnabled = true
            btnAll_OFF.isEnabled = true
        }
        
        tsService.setTimePins(pins: timePins as! [Int])
        //checkButtonStates()
        
    }
    
    override func traitCollectionDidChange(_ previousTraitCollection: UITraitCollection?) {
        super.traitCollectionDidChange(previousTraitCollection)
        if self.traitCollection.verticalSizeClass == .compact && self.traitCollection.horizontalSizeClass == .compact && previousTraitCollection?.verticalSizeClass != .compact{
            for i in allButtonSettings {
                i?.changeToSmallButtonImages()
            }
        }
        if self.traitCollection.verticalSizeClass == .regular && previousTraitCollection?.verticalSizeClass == .compact {
            for i in allButtonSettings {
                i?.changeToBigButtonImages()
            }
        }
    }
 
    @IBAction func btnAll_ON_pressed(_ sender: UIButton) {
        btnAll_ON.backgroundColor = UIColor.green
        btnAll_OFF.backgroundColor = .clear
        for i in allButtonSettings {
            i?.switchToOn()
        }
        // set PowerSocketPin to set all on in Service
        psService.setPowerSocketPin(pin: 255, state: "ON")
    }
    
    @IBAction func btnAll_OFF_pressed(_ sender: UIButton) {
        btnAll_OFF.backgroundColor = UIColor.red
        btnAll_ON.backgroundColor = .clear
        for i in allButtonSettings {
            i?.switchToOff()
        }
        // set PowerSocketPin to set all off in Service
        psService.setPowerSocketPin(pin: 255, state: "OFF")
    }
    
    @IBAction func btnDisconnect_pressed(_ sender: UIButton) {
        // Disconnect from Device
        bluetoothManager.disconnectDevice()
        // back to ScanController
        print("switch back to Scan Controller...")
        backToScanController()
        self.presentingViewController?.dismiss(animated: true, completion: nil)
    }
    
    @IBAction func btnOpenSettings(_ sender: UIButton) {
        //ctService.setCurrentTime()
        if let svc = UIStoryboard(name: "Main", bundle: nil).instantiateViewController(withIdentifier: "SettingsController") as? SettingsViewController {
            print("Switch to SettingsViewController...")
            //svc.setBluetoothManager(manager: bluetoothManager)
            svc.setTSService(service: tsService)
            self.show(svc, sender: nil)
        } else {
            print("Could not instanciate SettingsController")
        }
    }
    
    @IBAction func btnPS_1_pressed(_ sender: UIButton) {
        allButtonSettings[0]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS1_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[0]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS1_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[0]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_2_pressed(_ sender: UIButton) {
        allButtonSettings[1]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS2_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[1]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS2_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[1]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_3_pressed(_ sender: UIButton) {
        allButtonSettings[2]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS3_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[2]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS3_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[2]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_4_pressed(_ sender: UIButton) {
        allButtonSettings[3]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS4_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[3]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS4_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[3]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_5_pressed(_ sender: UIButton) {
        allButtonSettings[4]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS5_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[4]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS5_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[4]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_6_pressed(_ sender: UIButton) {
        allButtonSettings[5]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS6_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[5]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS6_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[5]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS_7_pressed(_ sender: UIButton) {
        allButtonSettings[6]?.buttonAction(sender)
        checkButtonStates()
    }
    
    @IBAction func btnPS7_Man_pressed(_ sender: UIButton!) {
        allButtonSettings[6]?.radioButtonAction(sender, isManually: true)
        checkForRadioSettings()
    }
    
    @IBAction func btnPS7_Auto_pressed(_ sender: UIButton!) {
        allButtonSettings[6]?.radioButtonAction(sender, isManually: false)
        checkForRadioSettings()
    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */

}
