//
//  CurrentTimeService.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 29.10.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import CoreBluetooth

class CurrentTimeService {
    var ctService: CBService? = nil
    
    private var writeTimeCharacteristic: CBCharacteristic!
    
    init() {
    }
    
    public func discoverService(service: CBService) {
        if ctService == nil {
            ctService = service
            print("CurrentTimeService is available")
        } else {
            print("CurrentTimeService is set")
        }
    }
    
    public func getService() -> CBService {
        return ctService!
    }
    
    public func discoverCharacteristic()  {
        writeTimeCharacteristic = getCharacteristic(service: ctService!, charUUID: CurrentTimeServiceUUID.CHAR_CT_WRITE)
        if writeTimeCharacteristic != nil {
            print("CurrentTimeCharacteristic is set")
        }
        NotificationCenter.default.post(name: RCNotification.CurrentTimeCharacteristicsScanComplete, object: nil)
        // Add notification here
    }
    
    private func getCharacteristic(service: CBService, charUUID: CBUUID) -> CBCharacteristic {
        let c : [CBCharacteristic]? = service.characteristics
        var returnChar: CBCharacteristic?
        if c == nil {
            print("No characteristics found")
        } else {
            for char in c! {
                if char.uuid == charUUID {
                    returnChar = char
                }
            }
        }
        
        return returnChar!
    }
    
    public func setCurrentTime() {
        if ctService != nil {
            var val: [UInt8]? = nil
            let date = Date()
            let calendar = Calendar.current
            let components = calendar.dateComponents([.year, .month, .day, .hour, .minute, .second, .weekday], from: date)
            
            //let year: Int = components.year!
            let yearArr = String(components.year!).compactMap{Int(String($0))}
            let year1 = String(yearArr[0])
            let year2 = String(yearArr[1])
            let year3 = String(yearArr[2])
            let year4 = String(yearArr[3])
            let firstYear: String = year1 + year2
            let secYear: String = year3 + year4
            let month: Int = components.month!
            let day: Int = components.day!
            let hour: Int = components.hour!
            let minute: Int = components.minute!
            let seconds: Int = components.second!
            let weekday: Int = components.weekday! == 1 ? 7 : components.weekday! - 1
            
            val = [UInt8(Int(firstYear)!), UInt8(Int(secYear)!), UInt8(month), UInt8(day), UInt8(hour), UInt8(minute), UInt8(seconds), UInt8(weekday)]
            
            let value : [AnyHashable: Any]  = [
                AnyHashable("time"): val ?? [0x14, 0x12, 0x0A, 0x08, 0x10, 0x23, 0x0A]
            ]
            print(value)
            print("setCurrentTime")
            
            NotificationCenter.default.post(name: RCNotification.CurrentTimeWriteCharacteristicValue, object: nil, userInfo: value)
        }
    }
    
    public func getWriteCharacteristic() -> CBCharacteristic {
        return writeTimeCharacteristic
    }
    
    public func disconnect() {
        ctService = nil
        writeTimeCharacteristic = nil
    }
}
