//
//  PowerSocketService.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 04.09.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import CoreBluetooth

class PowerSocketService {
    
    var psService: CBService? = nil
    
    private var pinWriteCharacteristic: CBCharacteristic!
    private var pinReadCharacteristic: CBCharacteristic!
    
    init() {
    }
    
    public func discoverService(service: CBService) {
        if psService == nil {
            psService = service
            print("PowerSocketService is available")
        } else {
            print("PowerSocketService is set")
        }
    }
    
    public func getService() -> CBService {
        return psService!
    }
    
    public func discoverCharacteristics() {
        // Extract characteristics from service
        pinWriteCharacteristic = getCharacteristic(service: psService!, charUUID: PowerSocketUUID.CHAR_PS_PIN_WRITE)
        pinReadCharacteristic = getCharacteristic(service: psService!, charUUID: PowerSocketUUID.CHAR_PS_PIN_READ)
        if pinWriteCharacteristic != nil && pinReadCharacteristic != nil {
            print("Both Characteristics are set")
        } else {
            print("Not both characteristics set")
        }
        NotificationCenter.default.post(name: RCNotification.PowerSocketCharacteristicsScanComplete, object: nil)
    }
    
    private func getCharacteristic(service: CBService, charUUID: CBUUID) -> CBCharacteristic {
        let c : [CBCharacteristic]? = service.characteristics
        var returnChar: CBCharacteristic?
        if c == nil {
            print("No characteristics found")
        } else {
            for char in c! {
                if char.uuid == charUUID {
                    returnChar = char
                }
            }
        }
        return returnChar!
    }
    
    public func setPowerSocketPin(pin: Int, state: String) {
        if psService != nil {
            var val: [UInt8]? = nil
            switch pin {
            case 1:
                if state == "ON" {
                    val = [0x00, 0x01]
                } else if state == "OFF" {
                    val = [0x00, 0x00]
                }
                break
            case 2:
                if state == "ON" {
                    val = [0x01, 0x01]
                } else if state == "OFF" {
                    val = [0x01, 0x00]
                }
                break
            case 3:
                if state == "ON" {
                    val = [0x02, 0x01]
                } else if state == "OFF" {
                    val = [0x02, 0x00]
                }
                break
            case 4:
                if state == "ON" {
                    val = [0x03, 0x01]
                } else if state == "OFF" {
                    val = [0x03, 0x00]
                }
                break
            case 5:
                if state == "ON" {
                    val = [0x04, 0x01]
                } else if state == "OFF" {
                    val = [0x04, 0x00]
                }
                break
            case 6:
                if state == "ON" {
                    val = [0x05, 0x01]
                } else if state == "OFF" {
                    val = [0x05, 0x00]
                }
                break
            case 7:
                if state == "ON" {
                    val = [0x06, 0x01]
                } else if state == "OFF" {
                    val = [0x06, 0x00]
                }
                break
            case 255:
                if state == "ON" {
                    val = [0xFF, 0xFF]
                } else if state == "OFF" {
                    val = [0xFF, 0x00]
                }
                break
            default:
                break
            }
            let value: [AnyHashable: Any] = [
                AnyHashable("value"): val ?? [0x99, 0x99]
            ]
            // Sent Notification on Bluetooth Manager to write the value
            NotificationCenter.default.post(name: RCNotification.PowerSocketWriteCharacteristicValue, object: nil, userInfo: value)
        }
    }
    
    public func getWriteCharacteristic() -> CBCharacteristic {
        return pinWriteCharacteristic
    }
    
    public func getReadCharacteristic() -> CBCharacteristic {
        return pinReadCharacteristic
    }
    
    public func disconnect() {
        psService = nil
        pinWriteCharacteristic = nil
        pinReadCharacteristic = nil
    }
}
