//
//  TimeSettingsService.swift
//  iOS_ble_app
//
//  Created by Nicolas Wiedmer on 29.10.18.
//  Copyright © 2018 Nicolas Wiedmer. All rights reserved.
//

import CoreBluetooth

enum Bit: UInt8 {
    case zero, one
    
    var description: Int {
        switch self {
        case .one:
            return 1
        case .zero:
            return 0
        }
    }
}

class TimeSettingsService {
    
    var tsService: CBService? = nil
    
    private var writeTimeSettingsCharacteristic: CBCharacteristic!
    private var writeTimePinsCharacteristic: CBCharacteristic!
    
    init(){
    }
    
    public func discoverService(service: CBService) {
        if tsService == nil {
            tsService = service
            print("TimeSettingsService is available")
        } else {
            print("TimeSettingsService is set")
        }
    }
    
    public func getService() -> CBService {
        return tsService!
    }
    
    public func discoverCharacteristic() {
        writeTimeSettingsCharacteristic = getCharacteristic(service: tsService!, charUUID: TimeSettingsServiceUUID.CHAR_TS_WRITE)
        writeTimePinsCharacteristic = getCharacteristic(service: tsService!, charUUID: TimeSettingsServiceUUID.CHAR_TS_WRITE_PINS)
        if writeTimeSettingsCharacteristic != nil && writeTimePinsCharacteristic != nil{
            print("Both Time Setting Characteristics are found")
        }
        NotificationCenter.default.post(name: RCNotification.TimeSettingsCharacteristicsScanComplete, object: nil)
    }
    
    private func getCharacteristic(service: CBService, charUUID: CBUUID) -> CBCharacteristic {
        let c : [CBCharacteristic]? = service.characteristics
        var returnChar: CBCharacteristic?
        if c == nil {
            print("No characteristics found")
        } else {
            for char in c! {
                if char.uuid == charUUID {
                    returnChar = char
                }
            }
        }
        return returnChar!
    }
    
    public func setTimePins(pins: [Int]) {
        var pinBytes : [UInt8] = []
        for pin in pins {
            let byte = UInt8(pin)
            print(byte)
            pinBytes.append(byte)
        }
        
        let value : [AnyHashable : Any] = [
            AnyHashable("timePins"): pinBytes
        ]
        
        NotificationCenter.default.post(name: RCNotification.TimeSettingsWritePinCharacteristicValue, object: nil, userInfo: value)
    }
    
    public func setTimeSettings(settings: String) {
        //print("setTimeSettings: " + settings)
        let allSettings = settings.components(separatedBy: ";")
        var bits : [[Bit]] = []
        for item in allSettings {
            if item != "" {
                var subSetting = item.components(separatedBy: ",")
                if Int(subSetting[0])! % 7 == 0 {
                    if Int(subSetting[0])! != 0 {
                        bits[bits.endIndex - 1].append(.zero)
                    }
                    bits.append([])
                    print("new Line " + subSetting[0])
                }
                if Int(subSetting[0])! == 167 {
                    bits[bits.endIndex - 1].append(.zero)
                }
                if subSetting[1] == "ON" {
                    bits[bits.endIndex - 1].append(.zero)
                } else {
                    bits[bits.endIndex - 1].append(.one)
                }
            }
        }
        print(bits)
        var bytes : [UInt8] = []
        for i in 0..<24 {
            let a = bits[i][0].description * 1
            let b = bits[i][1].description * 2
            let c = bits[i][2].description * 4
            let d = bits[i][3].description * 8
            let e = bits[i][4].description * 16
            let f = bits[i][5].description * 32
            let g = bits[i][6].description * 64
            let h = bits[i][7].description * 128
            let intVal = a + b + c  + d + e + f + g + h
            let byte = UInt8(intVal)
            bytes.append(byte)
        }
        print(bytes)
        
        let value : [AnyHashable : Any] = [
            AnyHashable("settings"): bytes
        ]
        
        NotificationCenter.default.post(name: RCNotification.TimeSettingsWriteCharacteristicValue, object: nil, userInfo: value)
    }
    
    public func getWriteTimeCharacteristic() -> CBCharacteristic {
        return writeTimeSettingsCharacteristic
    }
    
    public func getWriteTimePinCharacteristic() -> CBCharacteristic {
        return writeTimePinsCharacteristic
    }
    
    public func disconnect() {
        tsService = nil
        writeTimeSettingsCharacteristic = nil
        writeTimePinsCharacteristic = nil
    }
}
